#!/bin/bash

set -ef -o pipefail

# This script installs python 2.7 for spearmint

SOURCE="${BASH_SOURCE[0]}"
while [ -h "$SOURCE" ]; do
  DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
  SOURCE="$(readlink "$SOURCE")"
  [[ $SOURCE != /* ]] && SOURCE="$DIR/$SOURCE"
done
DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"

# Use prefix dir if gaven
case $1 in
    --prefix=*)
        BASE_DIR=${1#*=}
    ;;
    '')
        BASE_DIR="$DIR"
    ;;
    *)
        echo "Wrong args: $1. Expect --prefix= or nothing."
        exit
    ;;
esac

#echo "$BASE_DIR will be used as base directory to install spearmint."

#BASE_DIR="$DIR/.."
PYTHON_DIR="$BASE_DIR/spearmint/lib/"
TMP_DIR="$PYTHON_DIR/src"

mkdir -p "$PYTHON_DIR"
mkdir -p "$TMP_DIR"

# Install OpenSSL from source
echo ""
echo "Installing OpenSSL 1.1.1d for Python2 ..."
cd "$TMP_DIR"

if [ ! -f "$PYTHON_DIR/bin/openssl" ]; then
    if [ ! -f "openssl-1.1.1d.tar.gz"  ]; then
        echo "Downloading the package ..."
        wget -q --no-check-certificate https://www.openssl.org/source/openssl-1.1.1d.tar.gz
    fi

    tar -zxf openssl-1.1.1d.tar.gz
    cd openssl-1.1.1d
    echo "Compiling ..."
    ./config --prefix="$PYTHON_DIR" --openssldir="$PYTHON_DIR/ssl" > /dev/null 2>&1
    make -j --quiet > /dev/null 2>&1
    echo "Installing ..."
    make install > /dev/null 2>&1
fi

# Install Zlib from source
echo ""
echo "Installing Zlib 1.2.11 for Python2 ..."
cd "$TMP_DIR"

if [ ! -f "$PYTHON_DIR/lib/zlib.a" ]; then
    if [ ! -f "zlib-1.2.11.tar.gz"  ]; then
        echo "Downloading the package ..."
        wget -q --no-check-certificate https://www.zlib.net/zlib-1.2.11.tar.gz
    fi

    tar -zxf zlib-1.2.11.tar.gz
    cd zlib-1.2.11
    echo "Compiling ..."
    ./configure --prefix="$PYTHON_DIR" > /dev/null 2>&1
    make -j --quiet > /dev/null 2>&1
    echo "Installing ..."
    make install > /dev/null 2>&1
fi

# used by python installation and pip
export LD_LIBRARY_PATH=$PYTHON_DIR/lib:$LD_LIBRARY_PATH


# Install Python 2.7.15 from source with ssl supporting
echo ""
echo "Installing Python 2.7.15 ..."
cd "$TMP_DIR"

if [ ! -x "$PYTHON_DIR/bin/python" ]; then
    if [ ! -f "Python-2.7.15.tgz"  ]; then
        echo "Downloading the package ..."
        wget -q --no-check-certificate http://www.python.org/ftp/python/2.7.15/Python-2.7.15.tgz
    fi

    tar -zxf Python-2.7.15.tgz
    cd Python-2.7.15

    echo "Compiling ..."
    ./configure --prefix="$PYTHON_DIR" --quiet > /dev/null 2>&1
    make -j --quiet > /dev/null 2>&1
    echo "Installing ..."
    make install > /dev/null 2>&1
fi

# Install pip
echo ""
echo "Installing pip2 ..."
cd "$TMP_DIR"

if [ ! -x "$PYTHON_DIR/bin/pip" ]; then
    if [ ! -f "get-pip.py" ]; then
        echo "Downloading the package ..."
        wget -q --no-check-certificate https://bootstrap.pypa.io/get-pip.py
    fi

    echo "Installing ..."
    "$PYTHON_DIR/bin/python" get-pip.py --no-cache-dir > /dev/null 2>&1
fi

# Install basic requirements first

cd "$BASE_DIR"
echo ""
echo "Installing required packages for Spearmint ..."
"$PYTHON_DIR/bin/pip" install -q --no-cache-dir --no-warn-script-location -r requirements_sp.txt > /dev/null 2>&1

# clean up
echo ""
echo "Cleaning up the temporary files ..."
rm -fr "$TMP_DIR"

cd "$BASE_DIR"

echo ""
echo "Complete"
echo ""

