"""Optimizer classes implement an algorithm exploring parameter ranges."""
from .basic import GridSearchOptimizer
from .harmonica import HarmonicaOptimizer
from .sp import SpearmintOptimizer
from .tune import TuneOptimizer


def get_optimizer(params, executor):
    """Given a Parameter object and Executor, return the specified optimizer."""
    if "optimizer" in params.vals:

        optimizer = params.vals['optimizer']

        if optimizer == "gridsearch":
            return GridSearchOptimizer(params, executor)

        if optimizer == "harmonica":
            return HarmonicaOptimizer(params, executor)

        if optimizer == "spearmint":
            return SpearmintOptimizer(params, executor)

        if optimizer == "ray_tune":
            return TuneOptimizer(params, executor)

        raise NotImplementedError("Requested optimizer '%s' is not supported" % optimizer)

    else:

        return GridSearchOptimizer(params, executor)
