from glob import glob
import os
import pytest
import shutil

from param_runner.param import ParamFile
from param_runner.executors import SrunExecutor, LocalExecutor
from param_runner.optimizers.basic import GridSearchOptimizer

class TestExecutors:
    # pytestmark = pytest.mark.skip

    @pytest.fixture(scope='session')
    def test_dir(self, tmpdir_factory):
        test_dir = tmpdir_factory.mktemp("param")
        return str(test_dir)

    @pytest.fixture(scope='session')
    def param_yaml(self, test_dir):
        test_param_source = os.path.dirname(
            os.path.abspath(__file__)) + '/test_data/test_params.yaml'
        shutil.copy(test_param_source, test_dir)
        return os.path.join(test_dir, 'test_params.yaml')

    @pytest.fixture(scope='session')
    def param_file(self, param_yaml):
        pf = ParamFile(param_yaml)
        pf.load()
        return pf

    def test_local_gridsearch(self, test_dir, param_file):
        le = LocalExecutor(test_dir, param_file)
        gs = GridSearchOptimizer(param_file, le)
        gs.run()

        # There should be 1 log directory
        log_dirs = glob(test_dir + '/param_runner*')
        assert len(log_dirs) == 1
        log_dir = log_dirs[0]

        # There should be 170 error files (one per task)
        err_files = glob(log_dir + '/*.err')
        assert len(err_files) == 170
        # All of the error files should be empty!
        for err_file in err_files:
            assert os.stat(err_file).st_size == 0

        # There should be 170 output files (one per task)
        out_files = glob(log_dir + '/*.out')
        assert len(out_files) == 170

        # There should be on trace.txt file
        trace_file = os.path.join(log_dir, 'trace.txt')
        assert os.path.exists(trace_file)
        # It should have 171 lines (header + 170 task summaries)
        with open(trace_file) as f:
            for i, l in enumerate(f):
                pass
        assert i+1 == 171

