from glob import glob
import os
import pytest
import shutil

from param_runner.param import ParamFile
from param_runner.executors import SlurmWrapperExecutor, LocalExecutor
from param_runner.optimizers.sp import SpearmintOptimizer

class TestSpearmint:
    # pytestmark = pytest.mark.skip

    @pytest.fixture(scope='session')
    def test_dir(self, tmpdir_factory):
        test_dir = tmpdir_factory.mktemp("param")
        return str(test_dir)

    @pytest.fixture(scope='session')
    def param_yaml(self, test_dir):
        # copy the yaml file to test dir
        test_param_source = os.path.dirname(
            os.path.abspath(__file__)) + '/test_data/test_params_spearmint.yaml'
        shutil.copy(test_param_source, test_dir)

        # copy the branin funciton file
        test_param_source = os.path.dirname(
            os.path.abspath(__file__)) + '/test_data/test_spearmint_braninpy.txt'
        shutil.copy(test_param_source, '%s/branin.py' % test_dir)

        # copy the configuration file for branin test
        test_param_source = os.path.dirname(
            os.path.abspath(__file__)) + '/test_data/test_spearmint_config.pb'
        shutil.copy(test_param_source, '%s/config.pb' % test_dir)

        return os.path.join(test_dir, 'test_params_spearmint.yaml')

    @pytest.fixture(scope='session')
    def param_file(self, param_yaml):
        pf = ParamFile(param_yaml)
        pf.load()
        return pf

    @pytest.mark.timeout(120)
    def test_slurm_spearmint(self, test_dir, param_file):
        sw = SlurmWrapperExecutor(test_dir, param_file)
        sp = SpearmintOptimizer(param_file, sw)

        # test for file missing errors
        shutil.move('%s/config.pb' % test_dir, '%s/config.pb.bak' % test_dir)
        assert sp.run() == False
        shutil.move('%s/config.pb.bak' % test_dir, '%s/config.pb' % test_dir)

        shutil.move('%s/branin.py' % test_dir, '%s/branin.py.bak' % test_dir)
        assert sp.run() == False
        shutil.move('%s/branin.py.bak' % test_dir, '%s/branin.py' % test_dir)

        # normal run
        assert sp.run() == True

        # There should be 1 log directory
        log_dirs = glob(test_dir + '/param_runner_20*')
        assert len(log_dirs) == 1
        log_dir = log_dirs[0]

        # There should be 3 error files
        err_files = glob(log_dir + '/*.err')
        assert len(err_files) == 3

        # There should be 3 output files
        out_files = glob(log_dir + '/*.out')
        assert len(out_files) == 3

        # There should be on trace.txt file
        trace_file = os.path.join(log_dir, 'trace.txt')
        assert os.path.exists(trace_file)
        # It should have 2 lines (header + 1 task summaries)
        with open(trace_file) as f:
            for i, l in enumerate(f):
                pass
        assert i+1 == 2

