import os
import tarfile
from setuptools import setup
from setuptools.command.build_py import build_py


class PreBuildCommand(build_py):
    """Post-installation for installation mode."""

    def __int__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def run(self):
        print('\nPacking up spearmint code ...\n')
        print(os.path.abspath('.'))
        sp_tgz = 'param_runner/lib/spearmint.tgz'
        if not os.path.exists(sp_tgz):
            tar = tarfile.open(sp_tgz, 'w:gz')
            tar.add('param_runner/lib/spearmint/',
                    filter=lambda x: None if x.name in ['param_runner/lib/spearmint/lib'] else x)
            tar.close()

        build_py.run(self)


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="param_runner",
    version="2.0.1a2",
    author="Peng Lian",
    author_email="peng.lian@utsouthwestern.edu",
    description="Parameter exploration on SLURM clusters.",
    long_description=read('README.md'),
    license="GPLv3",
    platforms=["Linux"],
    keywords=["hpc", "computing", "machine learning", "parameter optimization"],
    url="https://git.biohpc.swmed.edu/s190450/param_runner",
    packages=['param_runner', 'param_runner.executors', 'param_runner.optimizers'],
    include_package_data=True,
    exclude_package_data={
        'param_runner': ['lib/spearmint/lib/*']},
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
    ],
    install_requires=[
        'anyconfig==0.9.10',
        'jsonschema==3.2.0',
        'colorlog==4.1.0',
        'pyyaml==5.3',
        'yamllint==1.20.0',
        'pathos==0.2.5',
        'ray==0.8.5',
        'tensorflow==2.1.0',
        'tabulate',
        'requests',
        'psutil',
        'pytest',
        'pytest-timeout',
    ],
    cmdclass={'build_py': PreBuildCommand},
    scripts=['bin/param_runner'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest', 'pytest-cov'],
    zip_safe=False,
)

#install_requires=[l for l in open('requirements.txt').readlines() if len(l.strip())>0 or not l.strip().startswith('#')],
